import asyncio
import datetime
import io
import re
from importlib import import_module
from pathlib import Path

from clannotation.annotator import Annotator
import httpx
import lxml
from fastapi import HTTPException, Request, Response, UploadFile
from httpx import Response as HTTPXResponse
from lxml import etree
from lxml.etree import Element

import settings


class OpCit:
    def __init__(
        self,
        request: Request,
        instrumentation=None,
    ):
        self.request = request
        self.instrumentation = instrumentation
        self.namespace = ""

    @staticmethod
    def schema_error():
        """
        Return a Crossref validation error when the schema is not found
        :return: an XML string
        """
        response = (
            '<?xml version="1.0" encoding="UTF-8"?>'
            '<doi_batch_diagnostic status="completed" sp="cskAir.local">'
            '<record_diagnostic status="Failure" msg_id="4">'
            f"<msg>Record not processed because submitted XML does not match "
            f'any supported schema. Op Cit only supports 5.3.1"</msg>'
            "</record_diagnostic>"
            "<batch_data>"
            "<record_count>1</record_count>"
            "<success_count>0</success_count>"
            "<warning_count>0</warning_count>"
            "<failure_count>1</failure_count>"
            "</batch_data>"
            "</doi_batch_diagnostic>"
        )

        return response

    @staticmethod
    async def determine_successful_deposits(xml_response) -> list[str]:
        """
        Read back the XML response and return a list of DOIs that have been
        processed successfully.
        :param xml_response: the XML response from the live API.
        :return: a list of DOIs that have been processed successfully.
        """
        valid_statuses = {"Success", "Warning"}
        from lxml import etree

        try:
            parser = etree.XMLParser(recover=True)
            root = etree.fromstring(xml_response, parser=parser)

            dois = []
            for record_diagnostic in root.findall(".//record_diagnostic"):
                status = record_diagnostic.get("status")

                doi_element = (
                    record_diagnostic.find("doi")
                    if status in valid_statuses
                    else None
                )
                if doi_element is not None:
                    dois.append(doi_element.text)

            return dois
        except etree.XMLSyntaxError:
            return []

    async def submit_and_validate(
        self, file, login_id, login_password, request: Request, xml_io
    ):
        """
        Submit the XML to the live DOI registration service
        :param file: the file to submit
        :param login_id: the login ID
        :param login_password: the login password
        :param request: the Starlette request object
        :param xml_io: the XML IO object
        :return:
        """
        uf = {"mdFile": (file.filename, xml_io)}

        # submit the XML to the live DOI registration service
        response: HTTPXResponse = await self.proxy_deposit(
            request=request,
            files=uf,
            login_id=login_id,
            login_passwd=login_password,
        )

        response_body = await response.aread()

        # determine which deposits succeeded
        successful_deposits = await self.determine_successful_deposits(
            xml_response=response_body
        )

        return response, response_body, successful_deposits

    async def proxy_deposit(
        self, request: Request, files, login_id, login_passwd
    ) -> Response:
        """
        Send a deposit to the live API
        :param request: the request object
        :param files: the files to deposit
        :param login_id: the login credential
        :param login_passwd: the login password
        :return: httpx Response object
        """
        client = httpx.AsyncClient(
            base_url="https://doi.crossref.org/v2/", timeout=30
        )

        url = httpx.URL(
            path="deposits", query=request.url.query.encode("utf-8")
        )

        mut_headers = request.headers.mutablecopy()
        del mut_headers["content-length"]

        data = {
            "login_id": login_id,
            "login_passwd": login_passwd,
            "operation": "doMDUpload",
        }

        req = client.build_request(
            request.method,
            url,
            headers=mut_headers.raw,
            data=data,
            files=files,
        )

        self.log(f"Sending deposit request to {url}")

        r = await client.send(req, stream=True)

        self.log("Deposit request sent")

        return r

    async def _extract_and_validate_request(self):
        """
        Extract file, login_id, and login_password, and validate the request
        :return: a tuple of the file, login_id, login_password, etree_parsed
            and namespace
        """
        # extract the details and make sure they are a valid Crossref metadata
        # deposit request
        file, login_id, login_password = await self._validate_request(
            self.request
        )

        self.log("Request validated")

        etree_parsed, namespace = self._etree_and_namespace(file)

        (
            etree_object,
            is_valid,
            etree_parsed,
        ) = self._validate_xml_against_schema(parsed_xml=etree_parsed)

        if not is_valid:
            self.log(
                "Invalid XML submitted (does not conform to a "
                "supported Crossref schema)"
            )
            return (
                False,
                file,
                login_id,
                login_password,
                etree_parsed,
                namespace,
            )

        # if we get here, then the document is valid
        self.log("XML is valid Crossref schema")
        return True, file, login_id, login_password, etree_parsed, namespace

    async def _process_dois(self, etree_parsed, warnings) -> dict[str, Element]:
        """
        Process the DOIs in the XML
        :param etree_parsed: the parsed XML
        :param warnings: the warnings list
        :return: a dictionary of DOIs to journal articles
        """
        dois: list[Element] = await self._extract_dois(etree_parsed)
        self.log(f"Found {len(dois)} DOIs for processing")

        dois_to_journal_articles = {}

        for doi in dois:
            # first, find the parent journal_article element
            journal_article: Element = doi.getparent()

            # walk up the tree until we find the journal_article element
            while (
                journal_article is not None
                and not journal_article.tag.endswith("journal_article")
            ):
                journal_article: Element = journal_article.getparent()

                if journal_article is None:
                    break

            if journal_article is None:
                # top of the tree
                # error out
                self.log(f"{doi.text} is not a journal DOI")
                warnings.append(f"{doi.text} is not a journal DOI")
                continue
            else:
                self.log(f"Found journal article for {doi.text}")
                dois_to_journal_articles[doi.text] = journal_article

        return dois_to_journal_articles

    async def _extract_fields(self, journal_article, doi, warnings) -> tuple:
        """
        Extract the fields from the journal article
        :param journal_article: the journal article
        :param doi: the DOI
        :param warnings: the warnings list
        :return: a tuple of fields
        """
        # now look for open licensing information
        license_ref, license_regex = await self._extract_license(
            journal_article
        )

        is_cc = False

        # Extract the text content of the found license_ref
        if (
            license_ref
            and re.match(license_regex, license_ref[0].text) is not None
        ):
            self.log("License is a valid CC license")
            is_cc = True

            resource = await self._extract_fulltext(journal_article)

            if len(resource) > 0:
                remote_file = resource[0].text

                try:
                    self.log(f"Fetching {remote_file}")
                    file_object = await self._fetch_url(url=remote_file)
                except Exception as e:
                    message = (
                        f"Error fetching {remote_file}: {e}. "
                        "This item will not be archived."
                    )
                    self.log(message)
                    warnings.append(message)
                    file_object = None

                if file_object:
                    # extract authors
                    authors = await self._extract_authors(
                        journal_article, namespace=self.namespace
                    )

                    # extract the title
                    title = await self._extract_title(
                        journal_article, namespace=self.namespace
                    )

                    # extract the date
                    date = await self._extract_date(
                        journal_article, namespace=self.namespace
                    )
                else:
                    authors = None
                    title = None
                    date = None
                    resource = None
            else:
                message = (
                    f"No full text resource found for {doi}. "
                    "This item will not be archived."
                )

                self.log(message)
                warnings.append(message)

                file_object = None
                authors = None
                title = None
                date = None
                resource = None
        else:
            message = f"No valid CC license found for {doi}."

            self.log(message)
            warnings.append(message)

            file_object = None
            authors = None
            title = None
            date = None
            resource = None

        return (
            license_ref,
            license_regex,
            is_cc,
            resource,
            authors,
            title,
            date,
            file_object,
        )

    async def process(self) -> Response:
        """
        Process the deposit request
        :return: a Starlette Response object
        """
        warnings = []

        (
            valid,
            file,
            login_id,
            login_password,
            etree_parsed,
            namespace,
        ) = await self._extract_and_validate_request()

        if not valid:
            return Response(
                content=self.schema_error(),
                status_code=400,
            )

        # now look for DOIs
        dois_to_journal_articles = await self._process_dois(
            etree_parsed, warnings
        )

        for doi, journal_article in dois_to_journal_articles.items():
            # extract all fields from the journal article
            (
                license_ref,
                license_regex,
                is_cc,
                resource,
                authors,
                title,
                date,
                file_object,
            ) = await self._extract_fields(journal_article, doi, warnings)

            if is_cc and file_object:
                # deposit in archives
                for (
                    depositor_name,
                    depositor_module,
                ) in settings.Settings.DEPOSIT_SYSTEMS.items():
                    warnings, output = self.deposit(
                        depositor_name=depositor_name,
                        depositor_module=depositor_module,
                        file_object=file_object[0],
                        warnings=warnings,
                        doi=doi,
                        authors=authors,
                        title=title,
                        date=date,
                    )

                    # modify the journal article to point to the Op Cit resource
                    # and generate the resource JSON
                    return_json, md5_doi = await self._build_resource(
                        doi=doi,
                        output=output,
                        journal_article=journal_article,
                        title=title,
                        date=date,
                        full_tree=etree_parsed,
                    )

                    # push the JSON to S3
                    self.instrumentation.aws_connector.push_json_to_s3(
                        json_obj=return_json,
                        path=f"opcit/works/{md5_doi}.json",
                        bucket=settings.Settings.BUCKET,
                    )

                    self.instrumentation.logger.info(
                        f"Pushed {md5_doi} to S3 at opcit/works/{md5_doi}.json"
                    )

        print(etree.tostring(etree_parsed))

        # wrap a bytesio around etree_parsed
        xml_io = io.BytesIO(etree.tostring(etree_parsed))

        # deposit the XML in the Crossref API
        (
            response,
            response_body,
            successful_deposits,
        ) = await self.submit_and_validate(
            file=file,
            login_id=login_id,
            login_password=login_password,
            request=self.request,
            xml_io=xml_io,
        )

        response = Response(content=response_body)
        return response

    @staticmethod
    async def _generate_url(doi) -> (str, str):
        """
        Generate an OpCit URL from a DOI
        :param doi: the DOI from which to generate a URL
        :return: the URL
        """

        md5_doi = Annotator.doi_to_md5(doi)

        return f"https://api.labs.crossref.org/" f"opcit/{md5_doi}", md5_doi

    @staticmethod
    async def _build_resource(
        doi, output, journal_article, title, date, full_tree, namespace="5.3.1"
    ):
        return_json = {
            "doi": doi,
            "title": title,
            "resources": [output],
            "date": date.isoformat(),
        }

        # find all resources in the journal article

        resources = journal_article.xpath(
            "xmlns:doi_data/xmlns:resource",
            namespaces={"xmlns": f"http://www.crossref.org/schema/{namespace}"},
        )

        md5_doi = None

        for resource in resources:
            if resource.text not in return_json["resources"]:
                return_json["resources"].append(resource.text)
                resource.text, md5_doi = await OpCit._generate_url(doi)

        # see if we can find journal metadata in the parent element

        publication_title = full_tree.xpath(
            "//xmlns:journal_metadata/xmlns:full_title",
            namespaces={"xmlns": f"http://www.crossref.org/schema/{namespace}"},
        )

        return_json["publication_title"] = (
            publication_title[0].text if len(publication_title) > 0 else ""
        )

        return return_json, md5_doi

    @staticmethod
    async def _extract_date(
        journal_article, namespace="5.3.1"
    ) -> datetime.date:
        """
        Extract the date from the XML
        :param journal_article: the journal article
        :return: the date object
        """
        date_block: list[Element] = journal_article.xpath(
            ".//publication_date",
            namespaces={"xmlns": f"http://www.crossref.org/schema/{namespace}"},
        )

        date_object = datetime.date(
            year=int(
                journal_article.xpath(
                    ".//xmlns:publication_date/xmlns:year",
                    namespaces={
                        "xmlns": f"http://www.crossref.org/schema/{namespace}"
                    },
                )[0].text
            ),
            month=int(
                journal_article.xpath(
                    ".//xmlns:publication_date/xmlns:month",
                    namespaces={
                        "xmlns": f"http://www.crossref.org/schema/{namespace}"
                    },
                )[0].text
            ),
            day=int(
                journal_article.xpath(
                    ".//xmlns:publication_date/xmlns:day",
                    namespaces={
                        "xmlns": f"http://www.crossref.org/schema/{namespace}"
                    },
                )[0].text
            ),
        )

        return date_object

    @staticmethod
    async def _extract_title(journal_article, namespace="5.3.1") -> str:
        """
        Extract the title from the XML
        :param journal_article: the journal article
        :return: the title text
        """
        titles: list[Element] = journal_article.xpath(
            ".//xmlns:titles/xmlns:title",
            namespaces={"xmlns": f"http://www.crossref.org/schema/{namespace}"},
        )

        return titles[0].text

    @staticmethod
    async def _extract_authors(journal_article, namespace="5.3.1") -> list[str]:
        """
        Extract the authors from the XML
        :param journal_article: the journal article
        :return: a list of authors in FirstName LastName format
        """
        contributors: list[Element] = journal_article.xpath(
            ".//xmlns:contributors/*[@contributor_role = 'author']",
            namespaces={"xmlns": f"http://www.crossref.org/schema/{namespace}"},
        )

        authors = []

        for contributor in contributors:
            if contributor.tag.endswith("person_name"):
                author = {
                    "given_name": contributor.xpath(
                        ".//xmlns:given_name",
                        namespaces={
                            "xmlns": f"http://www.crossref.org/schema/{namespace}"
                        },
                    )[0].text,
                    "surname": contributor.xpath(
                        ".//xmlns:surname",
                        namespaces={
                            "xmlns": f"http://www.crossref.org/schema/{namespace}"
                        },
                    )[0].text,
                }

                is_first = contributor.xpath(
                    ".//@sequence[.='first']",
                    namespaces={
                        "xmlns": f"http://www.crossref.org/schema/{namespace}"
                    },
                )

                authenticated_orcid = contributor.xpath(
                    ".//xmlns:ORCID[@authenticated='true']",
                    namespaces={
                        "xmlns": f"http://www.crossref.org/schema/{namespace}"
                    },
                )

                if authenticated_orcid:
                    author["ORCID"] = authenticated_orcid[0].text

                if is_first:
                    authors.insert(
                        0, author["given_name"] + " " + author["surname"]
                    )
                else:
                    authors.append(
                        author["given_name"] + " " + author["surname"]
                    )

        return authors

    @staticmethod
    async def _extract_fulltext(journal_article, namespace="5.3.1"):
        """
        Extract the full text resource URL from the XML
        :param journal_article: the journal article element
        :return: a list of resource elements
        """
        resource: list[Element] = journal_article.xpath(
            './/xmlns:collection[@property="crawler-based"]/xmlns:item'
            '[@crawler="iParadigms"]/xmlns:resource',
            namespaces={"xmlns": f"http://www.crossref.org/schema/{namespace}"},
        )
        return resource

    @staticmethod
    async def _extract_license(journal_article):
        """
        Extract the license from the XML
        :param journal_article: the journal article
        :return: the license_ref element and the license_regex for matching
        """
        # this extracts license elements of the kind:
        # <license_ref applies_to="vor" start_date="2008-08-13">
        # http://creativecommons.org/licenses/by/3.0/deed.en_US</license_ref>

        license_regex = r"^https?://creativecommons.org/licenses/.+"
        license_element = "license_ref"
        necessary_applies_to = "vor"
        license_ref = journal_article.xpath(
            f'//xmlns:{license_element}[@applies_to="'
            f'{necessary_applies_to}"]',
            namespaces={
                "xmlns": "http://www.crossref.org/AccessIndicators.xsd"
            },
        )
        return license_ref, license_regex

    @staticmethod
    async def _extract_dois(parsed_xml) -> list[Element]:
        """
        Extract the DOIs from the XML
        :param parsed_xml: the parsed XML
        :return: a list of DOI elements
        """
        crossref_namespace = OpCit._extract_namespace(parsed_xml)
        namespaces = {"crossref": crossref_namespace}

        doi_data_elements = parsed_xml.findall(
            f".//crossref:doi_data/crossref:doi", namespaces=namespaces
        )
        return doi_data_elements

    def deposit(
        self,
        depositor_name,
        depositor_module,
        file_object,
        warnings: list,
        doi,
        authors,
        title,
        date,
    ) -> tuple[list, str]:
        """
        Deposit a file in an archive
        :param doi: the DOI
        :param warnings: the warnings list
        :param depositor_name: the name
        :param depositor_module: depositor module string
        :param file_object: the file object to deposit
        :param authors: the authors
        :param date: the item's date
        :return: a list of warnings and an output location string
        """
        self.log(f"Loading module {depositor_name}")
        process_module = import_module(f"depositors.{depositor_module}")

        # build a metadata dictionary
        metadata = {
            "title": title,
            "authors": authors,
            "date": date,
            "doi": doi,
        }

        archive = process_module.Archive(self.log)
        warnings, output = archive.deposit(
            file_object=file_object,
            warnings=warnings,
            doi=doi,
            metadata=metadata,
        )

        self.log(f"Deposited {doi} to {depositor_name} at {output}")

        return warnings, output

    def log(self, message) -> None:
        """
        Log a message
        :param message: the message to log
        :return: None
        """
        if self.instrumentation:
            self.instrumentation.logger.info(message)

    @staticmethod
    async def _make_request(client: httpx.AsyncClient, url: str) -> bytes:
        """
        Make a remote request
        :param client: the client object
        :param url: the URL to request
        :return: the response
        """
        response = await client.get(url)

        return response.content

    @staticmethod
    async def _fetch_url(url) -> tuple[bytes]:
        """
        Fetch a remote URL
        :param url: the URL to fetch
        :return: list of async task responses
        """
        async with httpx.AsyncClient() as client:
            tasks = [OpCit._make_request(client, url)]
            result = await asyncio.gather(*tasks)
            return result

    @staticmethod
    async def _validate_request(request) -> tuple[UploadFile, str, str]:
        """
        Check that this is a valid deposit request and extract the needed info
        :param request: the current request
        :return: a tuple of the submitted file, login_id, and login_passwd
        """
        # 1. check that the content type of this request is
        #  "multipart/form-data"
        # 2. check that the following parameters are all set:
        # a. login_id, the depositing username and/or role in format user/role
        #  or role
        # b. login_passwd, the depositing user password
        # c. operation, the value should be “doMDUpload”.
        # 3. extract the file from the request (in the "files" parameter)

        content_type = request.headers.get("content-type")
        if content_type.startswith("multipart/form-data"):
            form = await request.form()

            if (
                "login_id" in form
                and "login_passwd" in form
                and "operation" in form
            ):
                if form["operation"] == "doMDUpload":
                    if "mdFile" in form:
                        return (
                            form["mdFile"],
                            form["login_id"],
                            form["login_passwd"],
                        )
                    else:
                        raise HTTPException(
                            status_code=400,
                            detail="Missing 'mdFile' field in the request.",
                        )
                else:
                    raise HTTPException(
                        status_code=400,
                        detail="Invalid value for 'operation'. "
                        "Must be 'doMDUpload'.",
                    )
            else:
                raise HTTPException(
                    status_code=400,
                    detail="Missing required parameters "
                    "(login_id, login_passwd, operation).",
                )
        else:
            raise HTTPException(
                status_code=400,
                detail="Invalid content type. Must be 'multipart/form-data'.",
            )

    def _validate_xml_against_schema(
        self,
        parsed_xml,
        base_url="",
        reload=False,
    ) -> tuple[etree, bool, etree]:
        """
        Validate the XML against the schema
        :param parsed_xml: the parsed XML
        :param base_url: the base URL
        :param reload: whether to reload the XML
        :return: tuple of etree, is_valid, parsed_xml
        """

        if isinstance(parsed_xml, lxml.etree._ElementTree):
            root = parsed_xml.getroot()
        else:
            root = parsed_xml

        schema_location = root.get(
            "{http://www.w3.org/2001/XMLSchema-instance}schemaLocation"
        )
        regex = r"^http://www.crossref.org/schema/(.+) "

        match = re.search(regex, schema_location)

        if match:
            capturing_group = match.group(1)
        else:
            return etree, False, parsed_xml

        self.namespace = match.group(1)

        xsd_file = Path(
            f"depositor_schema/{capturing_group}/crossref{capturing_group}.xsd"
        )

        xml_element_tree = parsed_xml

        if not xsd_file.exists():
            return etree, False, None

        if base_url:
            xsd_tree = etree.parse(
                xsd_file, base_url=base_url if base_url else None
            )
        else:
            xsd_tree = etree.parse(xsd_file)

        if reload:
            # reparse
            xml_element_tree = etree.fromstring(
                etree.tostring(xml_element_tree)
            )

        xml_schema = etree.XMLSchema(xsd_tree)
        is_valid = xml_schema.validate(xml_element_tree)

        return etree, is_valid, parsed_xml

    def _etree_and_namespace(self, file) -> tuple[etree, str]:
        """
        Parse the XML and extract the namespace
        :param file: the file to parse
        :return: tuple of etree and namespace
        """
        # validate the XML against the schema
        from lxml import etree

        parsed_xml = etree.parse(file.file)

        return parsed_xml, self._extract_namespace(parsed_xml)

    @staticmethod
    def _extract_namespace(
        tree, namespace: str = None, namespace_mode=False
    ) -> str:
        """
        Extract the namespace from an XML tree.
        :param tree: the XML tree
        :param namespace: the namespace or None
        :param namespace_mode: whether to return the namespace or the URL
        :return: the namespace
        """

        if namespace is None:
            return tree.xpath(f"namespace-uri(.)")

        else:
            ns_select = (
                "//namespace::*[not(. = "
                "'http://www.w3.org/XML/1998/namespace') "
                "and . = namespace-uri(..)]"
            )

            for ns, url in tree.xpath(ns_select):
                if not namespace_mode:
                    if ns == namespace:
                        return url
                else:
                    if url == namespace:
                        return ns



import json

from fastapi import Request, Response
from starlette.templating import Jinja2Templates

import settings


class Interstitial:
    def __init__(
        self,
        request: Request,
        instrumentation=None,
    ):
        self.request = request
        self.instrumentation = instrumentation

    async def run(self, identifier) -> Response:
        templates = Jinja2Templates(directory="../templates")

        # load the context from s3
        json_result = json.loads(
            self.instrumentation.aws_connector.s3_obj_to_str(
                bucket=settings.Settings.BUCKET,
                s3_path=f"opcit/works/{identifier}.json",
            )
        )

        json_result["request"] = self.request

        return templates.TemplateResponse(
            "chooser.html",
            json_result,
        )



import opcit.opcit.interstitial

try:
    import settings
    import deposit as deposit_system
except ImportError:
    import opcit.settings as settings
    import opcit.deposit as deposit_system

from fastapi import APIRouter, Request
from fastapi.templating import Jinja2Templates
from longsight.instrumentation import instrument

router = APIRouter()
templates = Jinja2Templates(directory="../templates")


@router.post("/deposit/preserve/", tags=["deposit", "preservation"])
@instrument(
    create_aws=True,
    bucket=settings.Settings.BUCKET,
    sign_aws_requests=True,
    cloudwatch_push=True,
    log_group_name=settings.Settings.LOG_STREAM_GROUP,
    log_stream_name=settings.Settings.LOG_STREAM_NAME,
)
async def deposit(
    request: Request,
    instrumentation=None,
):
    return await deposit_system.OpCit(
        request=request, instrumentation=instrumentation
    ).process()


@router.get("/opcit/{identifier}/", tags=["preservation"])
@instrument(
    create_aws=True,
    bucket=settings.Settings.BUCKET,
    sign_aws_requests=True,
    cloudwatch_push=True,
    log_group_name=settings.Settings.LOG_STREAM_GROUP,
    log_stream_name=settings.Settings.LOG_STREAM_NAME,
)
async def deposit(
    request: Request,
    instrumentation=None,
    identifier: str = None,
):
    return await opcit.opcit.interstitial.Interstitial(
        request=request, instrumentation=instrumentation
    ).run(identifier)



import hashlib
import sys
import unittest
from unittest import mock
from unittest.mock import MagicMock, patch, Mock


class TestDeposit(unittest.TestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    def log(self, message):
        return

    @patch("opcit.opcit.depositors.ia.time")
    def test_deposit_success(self, mock_time):
        """
        Test the deposit method with a successful deposit
        """
        # Mock the _check_mimes and _get_ia_session methods
        mock_check_mimes = Mock(return_value=(True, "application/pdf"))
        mock_get_ia_session = Mock()

        from opcit.opcit.depositors.ia import Archive

        instance = Archive(self.log)
        instance._check_mimes = mock_check_mimes
        instance._get_ia_session = mock_get_ia_session

        # Mock the return values for IAItem and its methods
        mock_ia_item = Mock()
        mock_ia_item.upload.return_value = Mock()
        mock_ia_item.urls.details = "https://example.com/item_details"

        mock_get_ia_session.return_value.get_item.return_value = mock_ia_item

        # Set the metadata and file object
        metadata = {"date": "2022-01-01"}
        file_object = b"File content"

        # Call the deposit method
        warnings, details_url = instance.deposit(
            file_object, [], "example_doi", metadata
        )

        # Assert that the mocks were called correctly
        mock_check_mimes.assert_called_once_with(file_object, [], "example_doi")
        mock_get_ia_session.assert_called_once()
        mock_time.time.assert_called_once()

        mock_ia_item.upload.assert_called_once_with(
            files={
                "{}-{}.pdf".format(
                    mock_time.time(),
                    hashlib.md5("example_doi".encode()).hexdigest(),
                ): mock.ANY
            },
            metadata=metadata,
        )
        mock_ia_item.modify_metadata.assert_called_once_with(metadata)

        self.assertEqual(warnings, [])
        self.assertEqual(details_url, "https://example.com/item_details")

    def test_deposit_mime_failure(self):
        """
        Test the deposit method with a MIME type failure
        """
        # Mock the _check_mimes method to return False
        mock_check_mimes = Mock(return_value=(False, "invalid/mime_type"))

        # Set up the mock instance of YourClass
        from opcit.opcit.depositors.ia import Archive

        instance = Archive(self.log)
        instance._check_mimes = mock_check_mimes

        # Call the deposit method
        warnings = instance.deposit(b"File content", [], "example_doi")

        # Assert that the mock was called correctly
        mock_check_mimes.assert_called_once_with(
            b"File content", [], "example_doi"
        )

        self.assertEqual(warnings, [])


class TestCheckMimes(unittest.TestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    def test_valid_pdf(self):
        """
        Test the _check_mimes function with a valid PDF file
        """
        # Simulate a valid PDF file by setting mime_type to "application/pdf"
        mime_type = "application/pdf"
        warnings = []

        # Call the _check_mimes function with the mock logger
        result, new_warnings = self._check_mimes(mime_type, warnings, "doi")

        # Assert that the function returns True for a valid PDF
        self.assertTrue(result)

        # Assert that the warnings list is unchanged
        self.assertEqual(new_warnings, [])

        # Assert that the logger was called with the correct message
        self.logger.assert_called_once_with(
            "File MIME type for doi is application/pdf"
        )

    def test_invalid_mime_type(self):
        """
        Test the _check_mimes function with an invalid MIME type
        """
        # Simulate an invalid MIME type
        mime_type = "image/jpeg"
        warnings = []

        # Call the _check_mimes function with the mock logger
        result, new_warnings = self._check_mimes(mime_type, warnings, "doi")

        # Assert that the function returns False for an invalid MIME type
        self.assertFalse(result)

        # Assert that the warnings list contains the expected warning message
        self.assertEqual(len(new_warnings), 1)
        expected_warning = (
            "File MIME type for doi is image/jpeg. This item will "
            "not be archived as only PDF is supported."
        )
        self.assertEqual(new_warnings[0], expected_warning)

    def _check_mimes(self, mime_type, warnings, doi):
        """
        Helper function to call the _check_mimes function with a mock logger
        :param mime_type: the MIME type to return from the mock
        :param warnings: the warnings list to pass to the function
        :param doi: the DOI to pass to the function
        :return: the result and new_warnings from the function
        """
        # Mock the 'magic' library's Magic class and its 'from_buffer' method
        from unittest.mock import patch

        with patch("magic.Magic") as mock_magic:
            mock_instance = mock_magic.return_value
            mock_instance.from_buffer.return_value = mime_type

            # Call the _check_mimes function with the mock logger
            from opcit.opcit.depositors import ia

            instance = ia.Archive(self.logger)
            result, new_warnings = instance._check_mimes(b"", warnings, doi)

        return result, new_warnings


class TestGetIaSession(unittest.TestCase):
    @patch("opcit.opcit.settings.Settings")
    @patch("opcit.opcit.depositors.ia.get_session")
    @patch("opcit.opcit.settings.json.loads")
    @patch("opcit.opcit.settings.claws.aws_utils.AWSConnector")
    def test_get_ia_session(
        self,
        mock_aws_connector,
        mock_json_loads,
        mock_get_session,
        mock_settings,
    ):
        """
        Test the _get_ia_session function
        """
        # Mock the settings.Settings.fetch_secrets method
        mock_settings.fetch_secrets.return_value = None

        # Mock the get_session method
        from internetarchive import ArchiveSession

        mock_session = MagicMock(spec=ArchiveSession)
        mock_get_session.return_value = mock_session

        # mock the settings access key function
        # Set up the mocks for the AWSConnector and json.loads methods
        mock_secret = (
            '{"access_key": "test_access_key", "secret_key": "test_secret_key"}'
        )
        mock_aws_connector.return_value.get_secret.return_value = mock_secret
        mock_json_loads.return_value = {
            "access_key": "test_access_key",
            "secret_key": "test_secret_key",
        }

        # Call the _get_ia_session function
        from opcit.opcit.depositors.ia import Archive

        result = Archive._get_ia_session()

        # Assert that the methods were called correctly

        mock_get_session.assert_called_once_with(
            config={
                "s3": {
                    "access": "test_access_key",
                    "secret": "test_secret_key",
                }
            }
        )
        self.assertEqual(result, mock_session)


if __name__ == "__main__":
    unittest.main()



import json
from collections import OrderedDict

import claws.aws_utils


class Settings:
    BUCKET = "outputs.research.crossref.org"
    SAMPLES_BUCKET = "samples.research.crossref.org"

    APP_NAME = "Op Cit Deposit System"
    ABOUT = "A digital preservation deposit system for Crossref's Labs API"
    VERSION = "0.0.1"
    MAILTO = "labs@crossref.org"
    LICENSE_INFO = {
        "name": "MIT License",
        "url": "https://opensource.org/licenses/MIT",
    }
    TERMS_OF_SERVICE = """
        Warnings, Caveats and Weasel Words:

        This is an experiment running on R&D equipment in a non-production environment.

        It may disappear without warning and/or perform erratically.

        If it isn’t working for some reason, come back later and try again.
    """

    SCHEME = "https"
    REPRESENTATION_VERSION = "1.0.0"

    HEADERS = {"User-Agent": "f{APP_NAME}; mailto:{MAILTO}"}

    LOG_STREAM_NAME = "op-cit-deposit"
    LOG_STREAM_GROUP = "op-cit"

    DEPOSIT_SYSTEMS = {"Internet Archive": "ia"}

    AWS_CONNECTOR = None
    ACCESS_KEY = {}

    @staticmethod
    def fetch_secrets():
        if Settings.ACCESS_KEY == {}:
            Settings.AWS_CONNECTOR = claws.aws_utils.AWSConnector(
                unsigned=False, bucket=Settings.BUCKET, region_name="us-east-1"
            )

            Settings.ACCESS_KEY = json.loads(
                Settings.AWS_CONNECTOR.get_secret("InternetArchiveS3LikeAPI")
            )



import datetime
import io
import sys
import unittest
from pathlib import Path
from string import Template
from typing import BinaryIO
from unittest.mock import MagicMock, patch, Mock, AsyncMock

import httpx
import lxml
from fastapi import HTTPException, UploadFile
from httpx import Response
from lxml import etree
from starlette.datastructures import FormData
from starlette.requests import Request


class TestProcessDOIs(unittest.IsolatedAsyncioTestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    async def test_process_dois(self):
        xml_content = """
            <doi_batch xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xsi:schemaLocation="http://www.crossref.org/schema/5.3.1 https://www.crossref.org/schemas/crossref5.3.1.xsd"
                xmlns="http://www.crossref.org/schema/5.3.1" xmlns:jats="http://www.ncbi.nlm.nih.gov/JATS1"
                xmlns:fr="http://www.crossref.org/fundref.xsd" version="5.3.1">
                <head>
                    <doi_batch_id>test.x</doi_batch_id>
                    <timestamp>${timestamp}</timestamp>
                    <depositor>
                        <depositor_name>Crossref</depositor_name>
                        <email_address>pfeeney@crossref.org</email_address>
                    </depositor>
                    <registrant>Society of Metadata Idealists</registrant>
                </head>
                <body>
                    <journal>
                        <journal_metadata language="en">
                            <full_title>Journal of Metadata Perfection</full_title>
                            <abbrev_title>JOMPer</abbrev_title>
                            <doi_data>
                                <doi>10.32013/487529</doi>
                                <resource>https://www.crossref.org/jomper</resource>
                            </doi_data>
                        </journal_metadata>
                        
                        <journal_article publication_type="full_text">
                            <titles>
                                <title>Why 'blah blah blah dah' is the second best test data title</title>
                            </titles>
                            <contributors>
                                <person_name sequence="first" contributor_role="author">
                                    <given_name>Minerva</given_name>
                                    <surname>Housecat</surname>
                                    <affiliations>
                                        <institution>
                                            <institution_id type="ror">https://ror.org/05gq02987</institution_id>
                                        </institution>
                                    </affiliations>
                                    <ORCID authenticated="true">https://orcid.org/0000-0002-4011-3590</ORCID>
                                </person_name>
                                <person_name sequence="additional" contributor_role="author">
                                    <given_name>Eve</given_name>
                                    <surname>Martin Paul</surname>
                                    <affiliations>
                                        <institution>
                                            <institution_id type="ror">https://ror.org/02mb95055</institution_id>
                                        </institution>
                                    </affiliations>
                                    <ORCID authenticated="true">https://orcid.org/0000-0002-5589-8511</ORCID>
                                </person_name>
                            </contributors>
                            <jats:abstract>
                                <jats:p>Agile
                                    best practices, thought leadership collective impact impact investing to
                                    families. And equal opportunity vibrant, the, storytelling synergy metadata
                                    matters B-corp unprecedented challenge. Venture philanthropy cultivate
                                    impact, state of play; white paper collaborative consumption entrepreneur
                                    collaborative cities inclusive. Parse empower communities movements
                                    targeted; radical; social enterprise issue outcomes big data venture
                                    philanthropy. </jats:p>
                            </jats:abstract>
                            <publication_date media_type="online">
                                <month>01</month>
                                <day>01</day>
                                <year>2022</year>
                            </publication_date>
                            <acceptance_date>
                                <month>05</month>
                                <day>21</day>
                                <year>2021</year>
                            </acceptance_date>
                                    <!-- CC BY license -->
                                    <program xmlns="http://www.crossref.org/AccessIndicators.xsd">
                                        <free_to_read/>
                                        <license_ref applies_to="tdm" start_date="2022-01-01"
                                            >https://creativecommons.org/licenses/by/4.0/</license_ref>
                                        <license_ref applies_to="vor" start_date="2022-01-01">https://creativecommons.org/licenses/by/4.0/</license_ref>
                                    </program>
            
                            <doi_data>
                                <doi>10.32013/123456789-23</doi>
                                <resource content_version="vor">https://www.crossref.org/xml-samples/</resource>
                                <!-- PDF URL for similarity check -->
                                <collection property="crawler-based">
                                    <item crawler="iParadigms">
                                        <resource mime_type="application/pdf">https://eprintsds.bbk.ac.uk/id/eprint/26645/1/9780198850489.pdf</resource>
                                    </item>
                                </collection>
                                <collection property="text-mining">
                                    <item>
                                        <resource content_version="vor" mime_type="text/xml"
                                            >https://www.crossref.org/example.xml</resource>
                                    </item>
                                </collection>
                            </doi_data>
                        </journal_article>
                    </journal>
                </body>
            </doi_batch>
        """

        # Create an Element object from the XML content
        etree_parsed = etree.fromstring(xml_content)

        from opcit.opcit import deposit

        instance = deposit.OpCit(request=Request(scope={"type": "http"}))

        warnings = []
        result = await instance._process_dois(etree_parsed, warnings)

        self.assertIsInstance(
            result["10.32013/123456789-23"], lxml.etree._Element
        )

    async def test_process_dois_no_journal_articles(self):
        xml_content = """
                    <doi_batch xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                        xsi:schemaLocation="http://www.crossref.org/schema/5.3.1 https://www.crossref.org/schemas/crossref5.3.1.xsd"
                        xmlns="http://www.crossref.org/schema/5.3.1" xmlns:jats="http://www.ncbi.nlm.nih.gov/JATS1"
                        xmlns:fr="http://www.crossref.org/fundref.xsd" version="5.3.1">
                        <head>
                            <doi_batch_id>test.x</doi_batch_id>
                            <timestamp>${timestamp}</timestamp>
                            <depositor>
                                <depositor_name>Crossref</depositor_name>
                                <email_address>pfeeney@crossref.org</email_address>
                            </depositor>
                            <registrant>Society of Metadata Idealists</registrant>
                        </head>
                        <body>
                            <journal>
                                <journal_metadata language="en">
                                    <full_title>Journal of Metadata Perfection</full_title>
                                    <abbrev_title>JOMPer</abbrev_title>
                                    <doi_data>
                                        <doi>10.32013/487529</doi>
                                        <resource>https://www.crossref.org/jomper</resource>
                                    </doi_data>
                                </journal_metadata>
                            </journal>
                        </body>
                    </doi_batch>
                """

        # Create an Element object from the XML content
        etree_parsed = etree.fromstring(xml_content)

        from opcit.opcit import deposit

        instance = deposit.OpCit(request=Request(scope={"type": "http"}))

        warnings = []
        result = await instance._process_dois(etree_parsed, warnings)

        self.assertEqual(result, {})


class TestExtractFields(unittest.IsolatedAsyncioTestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    async def test_extract_fields_with_license_and_no_resource(self):
        # Mock the dependencies of the function
        mock_license_ref = MagicMock()
        mock_license_ref.text = "https://creativecommons.org/licenses/by/4.0/"
        mock_license_regex = r"^https?://creativecommons.org/licenses/.+"
        mock_extract_license = AsyncMock(
            return_value=([mock_license_ref], mock_license_regex)
        )

        mock_fulltext_element = MagicMock()
        mock_fulltext_element.text = "https://example.com/fulltext.pdf"
        mock_extract_fulltext = AsyncMock(return_value=[mock_fulltext_element])

        mock_fetch_url = AsyncMock(return_value=b"File content")
        mock_fetch_url.side_effect = HTTPException(
            status_code=404, detail="Fulltext resource not found"
        )

        mock_extract_authors = AsyncMock(return_value=["Author 1", "Author 2"])
        mock_extract_title = AsyncMock(return_value="Title")
        mock_extract_date = AsyncMock(return_value="2022-01-01")

        # Set up the mock instance of YourClass
        from opcit.opcit import deposit

        instance = deposit.OpCit(request=Request(scope={"type": "http"}))

        instance._extract_license = mock_extract_license
        instance._extract_fulltext = mock_extract_fulltext
        instance._fetch_url = mock_fetch_url
        instance._extract_authors = mock_extract_authors
        instance._extract_title = mock_extract_title
        instance._extract_date = mock_extract_date

        # Call the extract_fields function
        journal_article = MagicMock()
        doi = "example_doi"
        warnings = []

        result = await instance._extract_fields(journal_article, doi, warnings)

        # Assert the function calls and the result
        mock_extract_license.assert_awaited_once_with(journal_article)
        mock_extract_fulltext.assert_awaited_once_with(journal_article)
        mock_fetch_url.assert_awaited_once_with(
            url="https://example.com/fulltext.pdf"
        )
        mock_extract_authors.assert_not_called()
        mock_extract_title.assert_not_called()
        mock_extract_date.assert_not_called()

        expected_result = (
            [mock_license_ref],
            mock_license_regex,
            True,
            None,
            None,
            None,
            None,
            None,
        )
        self.assertEqual(result, expected_result)

    async def test_extract_fields_with_license_and_resource(self):
        # Mock the dependencies of the function
        mock_license_ref = MagicMock()
        mock_license_ref.text = "https://creativecommons.org/licenses/by/4.0/"
        mock_license_regex = r"^https?://creativecommons.org/licenses/.+"
        mock_extract_license = AsyncMock(
            return_value=([mock_license_ref], mock_license_regex)
        )

        mock_fulltext_element = MagicMock()
        mock_fulltext_element.text = "https://example.com/fulltext.pdf"
        mock_extract_fulltext = AsyncMock(return_value=[mock_fulltext_element])

        mock_fetch_url = AsyncMock(return_value=b"File content")

        mock_extract_authors = AsyncMock(return_value=["Author 1", "Author 2"])
        mock_extract_title = AsyncMock(return_value="Title")
        mock_extract_date = AsyncMock(return_value="2022-01-01")

        # Set up the mock instance of YourClass
        from opcit.opcit import deposit

        instance = deposit.OpCit(request=Request(scope={"type": "http"}))

        instance._extract_license = mock_extract_license
        instance._extract_fulltext = mock_extract_fulltext
        instance._fetch_url = mock_fetch_url
        instance._extract_authors = mock_extract_authors
        instance._extract_title = mock_extract_title
        instance._extract_date = mock_extract_date

        # Call the extract_fields function
        journal_article = MagicMock()
        doi = "example_doi"
        warnings = []

        result = await instance._extract_fields(journal_article, doi, warnings)

        # Assert the function calls and the result
        expected_result = (
            [mock_license_ref],
            mock_license_regex,
            True,
            [mock_fulltext_element],
            ["Author 1", "Author 2"],
            "Title",
            "2022-01-01",
            b"File content",
        )
        self.assertEqual(result, expected_result)

    async def test_extract_fields_with_no_license_and_resource(self):
        # Mock the dependencies of the function
        mock_license_ref = MagicMock()
        mock_license_ref.text = "All Rights Reserved"
        mock_license_regex = r"^https?://creativecommons.org/licenses/.+"
        mock_extract_license = AsyncMock(
            return_value=([mock_license_ref], mock_license_regex)
        )

        mock_fulltext_element = MagicMock()
        mock_fulltext_element.text = "https://example.com/fulltext.pdf"
        mock_extract_fulltext = AsyncMock(return_value=[mock_fulltext_element])

        mock_fetch_url = AsyncMock(return_value=b"File content")

        mock_extract_authors = AsyncMock(return_value=["Author 1", "Author 2"])
        mock_extract_title = AsyncMock(return_value="Title")
        mock_extract_date = AsyncMock(return_value="2022-01-01")

        # Set up the mock instance of YourClass
        from opcit.opcit import deposit

        instance = deposit.OpCit(request=Request(scope={"type": "http"}))

        instance._extract_license = mock_extract_license
        instance._extract_fulltext = mock_extract_fulltext
        instance._fetch_url = mock_fetch_url
        instance._extract_authors = mock_extract_authors
        instance._extract_title = mock_extract_title
        instance._extract_date = mock_extract_date

        # Call the extract_fields function
        journal_article = MagicMock()
        doi = "example_doi"
        warnings = []

        result = await instance._extract_fields(journal_article, doi, warnings)

        # Assert the function calls and the result
        mock_extract_license.assert_awaited_once_with(journal_article)
        mock_extract_fulltext.assert_not_called()
        mock_fetch_url.assert_not_called()

        mock_extract_authors.assert_not_called()
        mock_extract_title.assert_not_called()
        mock_extract_date.assert_not_called()

        expected_result = (
            [mock_license_ref],
            mock_license_regex,
            False,
            None,
            None,
            None,
            None,
            None,
        )
        self.assertEqual(result, expected_result)


class TestExtractAndValidateRequest(unittest.IsolatedAsyncioTestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    @patch("opcit.opcit.deposit.OpCit._validate_request")
    @patch("opcit.opcit.deposit.OpCit._etree_and_namespace")
    @patch("opcit.opcit.deposit.OpCit._validate_xml_against_schema")
    async def test_extract_and_validate_request_valid(
        self, mock_validate_xml, mock_etree_and_namespace, mock_validate_request
    ):
        # Mock the necessary methods to simulate a valid request
        mock_file = MagicMock()
        mock_login_id = "user/role"
        mock_login_password = "password"
        mock_etree_parsed = MagicMock()
        mock_namespace = "http://www.crossref.org/schema/5.3.1"

        mock_validate_request.return_value = (
            mock_file,
            mock_login_id,
            mock_login_password,
        )
        mock_etree_and_namespace.return_value = (
            mock_etree_parsed,
            mock_namespace,
        )
        mock_validate_xml.return_value = (None, True, None)

        from opcit.opcit import deposit

        instance = deposit.OpCit(request=Request(scope={"type": "http"}))

        # Call the _extract_and_validate_request method
        result = await instance._extract_and_validate_request()

        # Assert that the result is a tuple with the expected values
        self.assertIsInstance(result, tuple)
        self.assertEqual(len(result), 6)
        self.assertTrue(result[0])  # Should be True for a valid request

    @patch("opcit.opcit.deposit.OpCit._validate_request")
    @patch("opcit.opcit.deposit.OpCit._etree_and_namespace")
    @patch("opcit.opcit.deposit.OpCit._validate_xml_against_schema")
    async def test_extract_and_validate_request_invalid_xml(
        self, mock_validate_xml, mock_etree_and_namespace, mock_validate_request
    ):
        # Mock the necessary methods to simulate an invalid XML request
        mock_file = MagicMock()
        mock_login_id = "user/role"
        mock_login_password = "password"
        mock_etree_parsed = MagicMock()
        mock_namespace = "http://www.crossref.org/schema/5.3.1"

        mock_validate_request.return_value = (
            mock_file,
            mock_login_id,
            mock_login_password,
        )
        mock_etree_and_namespace.return_value = (
            mock_etree_parsed,
            mock_namespace,
        )
        mock_validate_xml.return_value = (None, False, None)

        from opcit.opcit import deposit

        instance = deposit.OpCit(request=Request(scope={"type": "http"}))

        # Call the _extract_and_validate_request method
        result = await instance._extract_and_validate_request()

        # Assert that the result is a tuple with the expected values
        self.assertIsInstance(result, tuple)
        self.assertEqual(len(result), 6)
        self.assertFalse(result[0])  # Should be False for an invalid XML


class TestDeposit(unittest.TestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    @patch("opcit.opcit.deposit.import_module")
    def test_deposit_success(self, mock_import_module):
        # Mock the import_module to return a mock Archive class
        mock_archive_class = Mock()
        mock_import_module.return_value.Archive = mock_archive_class

        # Mock the deposit method of the Archive class
        mock_archive_instance = mock_archive_class.return_value
        mock_archive_instance.deposit.return_value = (None, None)

        from opcit.opcit import deposit

        instance = deposit.OpCit(request=Request(scope={"type": "http"}))

        # Call the deposit method with mock parameters
        depositor_name = "DepositorName"
        depositor_module = "DepositorModule"
        file_object = b"File content"
        warnings = []
        doi = "example_doi"

        instance.deposit(
            depositor_name=depositor_name,
            depositor_module=depositor_module,
            file_object=file_object,
            warnings=warnings,
            doi=doi,
            authors=[],
            title="",
            date="",
        )

        # Assert that the import_module and deposit methods were called
        mock_import_module.assert_called_once_with(
            f"depositors.{depositor_module}"
        )
        mock_archive_class.assert_called_once_with(instance.log)
        mock_archive_instance.deposit.assert_called_once_with(
            file_object=file_object,
            warnings=warnings,
            doi=doi,
            metadata={
                "authors": [],
                "date": "",
                "doi": "example_doi",
                "title": "",
            },
        )

    @patch("opcit.opcit.deposit.import_module")
    def test_deposit_import_error(self, mock_import_module):
        # Mock the import_module to raise an ImportError
        mock_import_module.side_effect = ImportError("Module not found")

        from opcit.opcit import deposit

        your_instance = deposit.OpCit(request=Request(scope={"type": "http"}))

        # Call the deposit method with mock parameters
        depositor_name = "DepositorName"
        depositor_module = "DepositorModule"
        file_object = b"File content"
        warnings = []
        doi = "example_doi"

        with self.assertRaises(ImportError):
            your_instance.deposit(
                depositor_name=depositor_name,
                depositor_module=depositor_module,
                file_object=file_object,
                warnings=warnings,
                doi=doi,
                authors=[],
                title="",
                date="",
            )


class TestFetchUrl(unittest.IsolatedAsyncioTestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    @patch("opcit.opcit.deposit.OpCit._make_request")
    async def test_fetch_url_success(self, mock_make_request):
        # Mock the _make_request function to return a response
        mock_response = Mock(spec=httpx.Response)
        mock_response.content = b"This is the content"
        mock_make_request.return_value = mock_response

        # Call the _fetch_url function with a mock URL
        from opcit.opcit import deposit

        url = "http://example.com"
        result: tuple[bytes] = await deposit.OpCit._fetch_url(url)

        # Assert that the result is a tuple containing bytes
        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0], Response)
        self.assertEqual(result[0].content, b"This is the content")

    @patch("opcit.opcit.deposit.OpCit._make_request")
    async def test_fetch_url_error(self, mock_make_request):
        # Mock the _make_request function to raise an exception
        mock_make_request.side_effect = httpx.RequestError("Error")

        # Call the _fetch_url function with a mock URL
        from opcit.opcit import deposit

        url = "http://example.com"
        result = None
        with self.assertRaises(httpx.RequestError):
            result = await deposit.OpCit._fetch_url(url)

        # Assert that the result is a tuple containing bytes
        self.assertIsNone(result)


class TestValidateXML(unittest.IsolatedAsyncioTestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    def test_validate_xml(self):
        # Create an XML tree without a namespace
        # load XML content from test.xml file and template in a timestamp

        import datetime

        current_time = datetime.datetime.now()

        time_stamp = str(current_time.timestamp()).split(".")[0]

        with open(
            "../../test_data/5.3.1.xml",
            "r",
        ) as f:
            xml = Template(f.read())
            xml = xml.substitute(timestamp=time_stamp)

        # Create an Element object from the XML content
        parsed_xml = etree.fromstring(xml.encode("utf-8"))

        # Call the _extract_namespace method without specifying a namespace
        from opcit.opcit import deposit

        opcit_obj = deposit.OpCit(request=Request(scope={"type": "http"}))

        (
            etree_obj,
            result,
            xml_returned,
        ) = opcit_obj._validate_xml_against_schema(parsed_xml)

        self.assertEqual(True, result)


class TestExtract(unittest.IsolatedAsyncioTestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    def test_extract_namespace_without_namespace(self):
        # Create an XML tree without a namespace
        xml_content = """
            <root>
                <element>Test</element>
            </root>
        """
        tree = etree.fromstring(xml_content)

        # Call the _extract_namespace method without specifying a namespace
        namespace = self._extract_namespace(tree)
        self.assertEqual(namespace, "")

    def _extract_namespace(self, tree, namespace=None, namespace_mode=False):
        from opcit.opcit import deposit

        # Call the _extract_namespace method
        return deposit.OpCit._extract_namespace(tree, namespace, namespace_mode)

    async def test_extract_title(self):
        # Sample XML content with a title
        xml_content = """
            <journal_article xmlns="http://www.crossref.org/schema/5.3.1">
                <titles>
                    <title>Title of the Article</title>
                </titles>
            </journal_article>
        """

        # Create an Element object from the XML content
        journal_article = etree.fromstring(xml_content)

        # Call the _extract_title method
        title = await self._extract_title(journal_article)

        # Assert that the title is extracted correctly
        self.assertEqual(title, "Title of the Article")

    @staticmethod
    async def _extract_title(journal_article):
        from opcit.opcit import deposit

        # Call the _extract_title method
        return await deposit.OpCit._extract_title(journal_article)

    async def test_extract_date(self):
        # Sample XML content with publication_date
        xml_content = """
            <journal_article xmlns="http://www.crossref.org/schema/5.3.1">
                <publication_date>
                    <year>2023</year>
                    <month>12</month>
                    <day>15</day>
                </publication_date>
            </journal_article>
        """

        # Create an Element object from the XML content
        journal_article = etree.fromstring(xml_content)

        # Call the _extract_date method
        date_object = await self._extract_date(journal_article)

        # Assert date_object is a datetime.date object with the expected date
        self.assertIsInstance(date_object, datetime.date)
        self.assertEqual(date_object, datetime.date(2023, 12, 15))

    @staticmethod
    def _extract_date(journal_article):
        from opcit.opcit import deposit

        # Call the _extract_date method
        return deposit.OpCit._extract_date(journal_article)

    async def test_extract_authors(self):
        # Sample XML content with authors
        xml_content = """
            <journal_article xmlns="http://www.crossref.org/schema/5.3.1">
                <contributors>
                    <person_name contributor_role="author">
                        <given_name>John</given_name>
                        <surname>Doe</surname>
                    </person_name>
                    <person_name contributor_role="author">
                        <given_name>Jane</given_name>
                        <surname>Smith</surname>
                        <ORCID authenticated="true">1234567890</ORCID>
                    </person_name>
                </contributors>
            </journal_article>
        """

        # Create an Element object from the XML content
        journal_article = etree.fromstring(xml_content)

        # Call the _extract_authors method
        authors = await self._extract_authors(journal_article)

        # Assert that authors are extracted correctly
        self.assertEqual(
            ["John Doe", "Jane Smith"],
            authors,
        )

    @staticmethod
    async def _extract_authors(journal_article):
        from opcit.opcit import deposit

        # Call the _extract_authors method
        return await deposit.OpCit._extract_authors(journal_article)

    async def test_extract_fulltext(self):
        # Sample XML content with fulltext resources
        xml_content = """
            <journal_article xmlns="http://www.crossref.org/schema/5.3.1">
                <collection property="crawler-based">
                    <item crawler="iParadigms">
                        <resource>Fulltext Resource 1</resource>
                        <resource>Fulltext Resource 2</resource>
                    </item>
                </collection>
            </journal_article>
        """

        # Create an Element object from the XML content
        journal_article = etree.fromstring(xml_content)

        # Mock the journal_article.xpath method

        resources = await self._extract_fulltext(journal_article)

        self.assertEqual(resources[0].text, "Fulltext Resource 1")
        self.assertEqual(resources[1].text, "Fulltext Resource 2")

    @staticmethod
    async def _extract_fulltext(journal_article):
        from opcit.opcit import deposit

        # Call the _extract_fulltext method
        return await deposit.OpCit._extract_fulltext(journal_article)

    async def test_extract_license(self):
        # Sample XML content with a license_ref element
        xml_content = """
            <journal_article xmlns="http://www.crossref.org/AccessIndicators.xsd">
                <license_ref applies_to="vor" start_date="2008-08-13">
                    http://creativecommons.org/licenses/by/3.0/deed.en_US
                </license_ref>
            </journal_article>
        """

        # Create an Element object from the XML content
        journal_article = etree.fromstring(xml_content)

        # Call the _extract_license method
        license_ref, license_regex = await self._extract_license(
            journal_article
        )

        # Assert that license_ref and license_regex are extracted correctly
        self.assertEqual(
            license_ref[0].text.strip(),
            "http://creativecommons.org/licenses/by/3.0/deed.en_US",
        )
        self.assertEqual(
            license_regex, "^https?://creativecommons.org/licenses/.+"
        )

    @staticmethod
    async def _extract_license(journal_article):
        from opcit.opcit import deposit

        # Call the _extract_license method
        return await deposit.OpCit._extract_license(journal_article)

    async def test_extract_dois(self):
        # Sample XML content with DOI data elements
        xml_content = """
            <doi_batch xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xsi:schemaLocation="http://www.crossref.org/schema/5.3.1 https://www.crossref.org/schemas/crossref5.3.1.xsd"
                xmlns="http://www.crossref.org/schema/5.3.1" xmlns:jats="http://www.ncbi.nlm.nih.gov/JATS1"
                xmlns:fr="http://www.crossref.org/fundref.xsd" version="5.3.1">
                <head>
                    <doi_batch_id>test.x</doi_batch_id>
                    <timestamp>${timestamp}</timestamp>
                    <depositor>
                        <depositor_name>Crossref</depositor_name>
                        <email_address>pfeeney@crossref.org</email_address>
                    </depositor>
                    <registrant>Society of Metadata Idealists</registrant>
                </head>
                <body>
                    <journal>
                        <journal_metadata language="en">
                            <full_title>Journal of Metadata Perfection</full_title>
                            <abbrev_title>JOMPer</abbrev_title>
                            <doi_data>
                                <doi>10.32013/487529</doi>
                                <resource>https://www.crossref.org/jomper</resource>
                            </doi_data>
                        </journal_metadata>
                    </journal>
                </body>
            </doi_batch>
        """

        # Create an Element object from the XML content
        parsed_xml = etree.fromstring(xml_content)

        # Call the _extract_dois method
        doi_data_elements = await self._extract_dois(parsed_xml)

        # Assert that doi_data_elements are extracted correctly
        self.assertEqual(len(doi_data_elements), 1)
        self.assertEqual(
            [element.text for element in doi_data_elements],
            ["10.32013/487529"],
        )

    @staticmethod
    async def _extract_dois(parsed_xml):
        from opcit.opcit import deposit

        # Call the _extract_dois method
        return await deposit.OpCit._extract_dois(parsed_xml)


class TestValidateRequest(unittest.IsolatedAsyncioTestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    @staticmethod
    async def return_form(form):
        return form

    @staticmethod
    def string_to_binary_io(data: str) -> BinaryIO:
        # Encode the string as bytes
        bytes_data = data.encode("utf-8")

        # Create a BinaryIO object using io.BytesIO
        binary_io = io.BytesIO(bytes_data)

        return binary_io

    async def test_valid_request(self):
        # Create a valid request with the required fields
        request = MagicMock()
        request.headers.get.return_value = "multipart/form-data"
        form_data = FormData(
            {
                "login_id": "user/role",
                "login_passwd": "password",
                "operation": "doMDUpload",
                "mdFile": UploadFile(
                    filename="test.txt", file=self.string_to_binary_io("test")
                ),
            },
        )

        request.form.return_value = self.return_form(form_data)

        # Call the _validate_request method
        file, login_id, login_passwd = await self._validate_request(request)

        # Assert that the extracted values are correct
        self.assertIsInstance(file, UploadFile)
        self.assertEqual(login_id, "user/role")
        self.assertEqual(login_passwd, "password")

    async def test_missing_mdFile(self):
        # Create a request with missing 'mdFile' field
        request = MagicMock()
        request.headers.get.return_value = "multipart/form-data"
        form_data = FormData(
            {
                "login_id": "user/role",
                "login_passwd": "password",
                "operation": "doMDUpload",
            },
        )

        request.form.return_value = self.return_form(form_data)

        # Call the _validate_request method and expect an HTTPException
        with self.assertRaises(HTTPException) as context:
            await self._validate_request(request)

        # Assert the expected error message
        self.assertEqual(context.exception.status_code, 400)
        self.assertEqual(
            context.exception.detail,
            "Missing 'mdFile' field in the request.",
        )

    async def test_invalid_operation(self):
        # Create a request with invalid 'operation' value
        request = MagicMock()
        request.headers.get.return_value = "multipart/form-data"
        form_data = FormData(
            {
                "login_id": "user/role",
                "login_passwd": "password",
                "operation": "Bad Op",
                "mdFile": UploadFile(
                    filename="test.txt", file=self.string_to_binary_io("test")
                ),
            },
        )

        request.form.return_value = self.return_form(form_data)

        # Call the _validate_request method and expect an HTTPException
        with self.assertRaises(HTTPException) as context:
            await self._validate_request(request)

        # Assert the expected error message
        self.assertEqual(context.exception.status_code, 400)
        self.assertEqual(
            context.exception.detail,
            "Invalid value for 'operation'. Must be 'doMDUpload'.",
        )

    async def test_missing_parameters(self):
        # Create a request with missing required parameters
        request = MagicMock()
        request.headers.get.return_value = "multipart/form-data"
        form_data = FormData(
            {
                "login_id": "user/role",
                "operation": "doMDUpload",
                "mdFile": UploadFile(
                    filename="test.txt", file=self.string_to_binary_io("test")
                ),
            },
        )

        request.form.return_value = self.return_form(form_data)

        # Call the _validate_request method and expect an HTTPException
        with self.assertRaises(HTTPException) as context:
            await self._validate_request(request)

        # Assert the expected error message
        self.assertEqual(context.exception.status_code, 400)
        self.assertEqual(
            context.exception.detail,
            "Missing required parameters (login_id, login_passwd, "
            "operation).",
        )

    async def test_invalid_content_type(self):
        # Create a request with an invalid content type
        request = MagicMock()
        request.headers.get.return_value = "application/json"
        form_data = FormData(
            {
                "login_id": "user/role",
                "login_passwd": "password",
                "operation": "doMDUpload",
                "mdFile": UploadFile(
                    filename="test.txt", file=self.string_to_binary_io("test")
                ),
            },
        )

        request.form.return_value = self.return_form(form_data)

        # Call the _validate_request method and expect an HTTPException
        with self.assertRaises(HTTPException) as context:
            await self._validate_request(request)

        # Assert the expected error message
        self.assertEqual(context.exception.status_code, 400)
        self.assertEqual(
            context.exception.detail,
            "Invalid content type. Must be 'multipart/form-data'.",
        )

    async def _validate_request(self, request):
        from opcit.opcit import deposit

        # Call the _validate_request method
        return await deposit.OpCit._validate_request(request)


class TestProcess(unittest.IsolatedAsyncioTestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    async def test_process_invalid_xml(self):
        # Mock the _extract_and_validate_request method to return invalid data
        from opcit.opcit.deposit import OpCit

        instance = OpCit(request=Request(scope={"type": "http"}))
        instance._extract_and_validate_request = AsyncMock(
            return_value=(False, None, None, None, None, None)
        )

        # Call the process method
        from starlette.responses import Response as StarletteResponse

        result: StarletteResponse = await instance.process()

        # Assert that the result is as expected
        self.assertIsInstance(result, StarletteResponse)
        self.assertEqual(
            result.body,
            b'<?xml version="1.0" encoding="UTF-8"?><doi_batch_diagnostic '
            b'status="completed" sp="cskAir.local"><record_diagnostic '
            b'status="Failure" msg_id="4"><msg>Record not processed because '
            b"submitted XML does not match any supported schema."
            b' Op Cit only supports 5.3.1"</msg></record_diagnostic>'
            b"<batch_data><record_count>1</record_count><success_count>0"
            b"</success_count><warning_count>0</warning_count>"
            b"<failure_count>1</failure_count></batch_data>"
            b"</doi_batch_diagnostic>",
        )
        self.assertEqual(result.status_code, 400)

    async def test_process_valid_xml(self):
        from opcit.opcit.deposit import OpCit

        instance = OpCit(request=Request(scope={"type": "http"}))
        instance._extract_and_validate_request = AsyncMock(
            return_value=(True, None, None, None, None, None)
        )
        instance._process_dois = AsyncMock(
            return_value={"doi1": MagicMock(), "doi2": MagicMock()}
        )

        # Mock the _extract_fields and deposit methods
        instance._extract_fields = AsyncMock(
            return_value=(None, None, False, [], None, None, None, None)
        )
        instance.deposit = MagicMock()

        # Call the process method
        from starlette.responses import Response as StarletteResponse

        result: StarletteResponse = await instance.process()

        # Assert that the result is as expected

        self.assertIsInstance(result, StarletteResponse)

        # TODO: this will need updating when we decide on the
        #  appropriate response
        self.assertEqual(result.body, b"Welcome to Op Cit")
        self.assertEqual(result.status_code, 200)

        # Assert that the _extract_fields and deposit methods were called
        instance._extract_fields.assert_called()
        self.assertEqual(instance._extract_fields.call_count, 2)
        instance.deposit.assert_not_called()


class TestRewrite(unittest.IsolatedAsyncioTestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    async def test_rewrite_journal_article(self):
        # Sample XML content with DOI data elements
        xml_content = """
            <journal_article xmlns="http://www.crossref.org/schema/5.3.1">
                <doi_data>
					<doi>10.32013/123456789-23</doi>
					<resource content_version="vor">https://www.crossref.org/xml-samples/</resource>
					<!-- PDF URL for similarity check -->
					<collection property="crawler-based">
						<item crawler="iParadigms">
							<resource mime_type="application/pdf">https://eprintsds.bbk.ac.uk/id/eprint/26645/1/9780198850489.pdf</resource>
						</item>
					</collection>
					<collection property="text-mining">
						<item>
                            <resource content_version="vor" mime_type="text/xml">https://www.crossref.org/example.xml</resource>
						</item>
					</collection>
				</doi_data>
            </journal_article>
        """
        # load another XML document from 5.3.1.xml using the Pathlib module
        xml_path = Path("../../test_data/5.3.1.xml")

        # Read the XML content from the file
        xml_content_full = etree.fromstring(xml_path.read_bytes())

        # Create an Element object from the XML content
        parsed_xml = etree.fromstring(xml_content)

        date = datetime.datetime.now()

        from opcit.opcit import deposit

        result, md5 = await deposit.OpCit._build_resource(
            doi="10.32013/123456789-23",
            output="https://www.test.com",
            journal_article=parsed_xml,
            title="Journal Title",
            date=date,
            full_tree=xml_content_full,
        )

        self.assertEqual(
            {
                "doi": "10.32013/123456789-23",
                "publication_title": "Journal of Metadata Perfection",
                "title": "Journal Title",
                "resources": [
                    "https://www.test.com",
                    "https://www.crossref.org/xml-samples/",
                ],
                "date": date.isoformat(),
            },
            result,
        )

        self.assertEqual(
            b'<journal_article xmlns="http://www.crossref.org/schema/5.3.1">\n                <doi_data>\n\t\t\t\t\t<doi>10.32013/123456789-23</doi>\n\t\t\t\t\t<resource content_version="vor">https://api.labs.crossref.org/opcit/ed5b0190fc96abbcc0c3c2b97f664b63</resource>\n\t\t\t\t\t<!-- PDF URL for similarity check -->\n\t\t\t\t\t<collection property="crawler-based">\n\t\t\t\t\t\t<item crawler="iParadigms">\n\t\t\t\t\t\t\t<resource mime_type="application/pdf">https://eprintsds.bbk.ac.uk/id/eprint/26645/1/9780198850489.pdf</resource>\n\t\t\t\t\t\t</item>\n\t\t\t\t\t</collection>\n\t\t\t\t\t<collection property="text-mining">\n\t\t\t\t\t\t<item>\n                            <resource content_version="vor" mime_type="text/xml">https://www.crossref.org/example.xml</resource>\n\t\t\t\t\t\t</item>\n\t\t\t\t\t</collection>\n\t\t\t\t</doi_data>\n            </journal_article>',
            etree.tostring(parsed_xml),
        )

    async def test_rewrite_journal_article_from_full(self):
        # load another XML document from 5.3.1.xml using the Pathlib module
        xml_path = Path("../../test_data/5.3.1.xml")

        # Read the XML content from the file
        xml_content_full = etree.fromstring(xml_path.read_bytes())

        # extract the first journal_article
        xml_element = xml_content_full.xpath(
            "//xmlns:journal_article",
            namespaces={"xmlns": f"http://www.crossref.org/schema/5.3.1"},
        )[0]

        date = datetime.datetime.now()

        from opcit.opcit import deposit

        result, md5 = await deposit.OpCit._build_resource(
            doi="10.32013/12345678-23",
            output="https://www.test.com",
            journal_article=xml_element,
            title="Journal Title",
            date=date,
            full_tree=xml_content_full,
        )

        self.assertEqual(
            {
                "doi": "10.32013/12345678-23",
                "publication_title": "Journal of Metadata Perfection",
                "title": "Journal Title",
                "resources": [
                    "https://www.test.com",
                    "https://www.crossref.org/xml-samples/",
                ],
                "date": date.isoformat(),
            },
            result,
        )

        self.assertEqual(
            b'<journal_article xmlns="http://www.crossref.org/schema/5.3.1" xmlns:xsi="htt'
            b'p://www.w3.org/2001/XMLSchema-instance" xmlns:jats="http://www.ncbi.nlm.nih.'
            b'gov/JATS1" xmlns:fr="http://www.crossref.org/fundref.xsd" publication_type="'
            b"full_text\">\n\t\t\t\t<titles>\n\t\t\t\t\t<title>Why 'blah blah blah' is"
            b" the best test data title</title>\n\t\t\t\t</titles>\n\t\t\t\t<contributor"
            b's>\n\t\t\t\t\t<person_name sequence="first" contributor_role="author">'
            b"\n\t\t\t\t\t\t<given_name>Minerva</given_name>\n\t\t\t\t\t\t<surname>da Ho"
            b"usecat</surname>\n\t\t\t\t\t\t<affiliations>\n\t\t\t\t\t\t\t<institutio"
            b'n>\n\t\t\t\t\t\t\t\t<institution_id type="ror">https://ror.org/05gq02987<'
            b"/institution_id>\n\t\t\t\t\t\t\t</institution>\n\t\t\t\t\t\t</affiliations>"
            b'\n\t\t\t\t\t\t<ORCID authenticated="true">https://orcid.org/0000-0002-4011-'
            b"3590</ORCID>\n\t\t\t\t\t</person_name>\n\t\t\t\t</contributors>\n\t\t\t\t<ja"
            b"ts:abstract>\n\t\t\t\t\t<jats:p>Agile\n\t\t\t\t\t\tbest practices, though"
            b"t leadership collective impact impact investing to\n\t\t\t\t\t\tfamilies. A"
            b"nd equal opportunity vibrant, the, storytelling synergy metadata\n\t\t\t"
            b"\t\t\tmatters B-corp unprecedented challenge. Venture philanthropy cultivat"
            b"e\n\t\t\t\t\t\timpact, state of play; white paper collaborative consumption"
            b" entrepreneur\n\t\t\t\t\t\tcollaborative cities inclusive. Parse empower co"
            b"mmunities movements\n\t\t\t\t\t\ttargeted; radical; social enterprise issue"
            b" outcomes big data venture\n\t\t\t\t\t\tphilanthropy. </jats:p>\n\t\t\t\t</j"
            b'ats:abstract>\n\t\t\t\t<publication_date media_type="online">\n\t\t\t\t\t<m'
            b"onth>05</month>\n\t\t\t\t\t<day>01</day>\n\t\t\t\t\t<year>2022</year>\n\t\t"
            b"\t\t</publication_date>\n\t\t\t\t<acceptance_date>\n\t\t\t\t\t<month>05</"
            b"month>\n\t\t\t\t\t<day>21</day>\n\t\t\t\t\t<year>2021</year>\n\t\t\t\t</a"
            b"cceptance_date>\n\t\t\t\t\t\t<!-- CC BY license -->\n\t\t\t\t\t\t<program "
            b'xmlns="http://www.crossref.org/AccessIndicators.xsd">\n\t\t\t\t\t\t\t<free_t'
            b'o_read/>\n\t\t\t\t\t\t\t<license_ref applies_to="tdm" start_date="2022-01-01'
            b'">https://creativecommons.org/licenses/by/4.0/</license_ref>\n\t\t\t\t\t\t\t'
            b'<license_ref applies_to="vor" start_date="2022-01-01">https://creativecommon'
            b"s.org/licenses/by/4.0/</license_ref>\n\t\t\t\t\t\t</program>\n\n\t\t\t\t<"
            b"doi_data>\n\t\t\t\t\t<doi>10.32013/12345678-23</doi>\n\t\t\t\t\t<resource co"
            b'ntent_version="vor">https://api.labs.crossref.org/opcit/ab7348a0f8fd34d56e56'
            b"26dee7b63714</resource>\n\t\t\t\t\t<!-- PDF URL for similarity check -->\n\t"
            b'\t\t\t\t<collection property="crawler-based">\n\t\t\t\t\t\t<item crawler="i'
            b'Paradigms">\n\t\t\t\t\t\t\t<resource mime_type="application/pdf">https://epr'
            b"ints.bbk.ac.uk/id/eprint/26645/1/9780198850489.pdf</resource>\n\t\t\t\t\t\t"
            b'</item>\n\t\t\t\t\t</collection>\n\t\t\t\t\t<collection property="text-minin'
            b'g">\n\t\t\t\t\t\t<item>\n\t\t\t\t\t\t\t<resource content_version="vor" mime'
            b'_type="text/xml">https://www.crossref.org/example.xml</resource>\n\t\t\t'
            b"\t\t\t</item>\n\t\t\t\t\t</collection>\n\t\t\t\t</doi_data>\n\t\t\t</journ"
            b"al_article>\n\n\t\t\t",
            etree.tostring(xml_element),
        )


class TestInterstitialPage(unittest.IsolatedAsyncioTestCase):
    def setUp(self):
        # Create a mock object for the logger
        self.logger = MagicMock()
        sys.path.append("../opcit/")

    async def test_generate_url(self):
        doi = "10.1234/example.doi"
        expected_url = "https://api.labs.crossref.org/opcit/md5_doi"
        expected_md5_doi = "md5_doi"

        import opcit.opcit.deposit

        with patch.object(
            opcit.opcit.deposit.Annotator,
            "doi_to_md5",
            return_value=expected_md5_doi,
        ):
            url, md5_doi = await opcit.opcit.deposit.OpCit._generate_url(doi)
            self.assertEqual(url, expected_url)
            self.assertEqual(md5_doi, expected_md5_doi)


if __name__ == "__main__":
    unittest.main()
